(*

   TAnotherGauge Component
   Version 1.0
   (C) 1996, Glen Why. No rights reserved

*)
unit AnotherGauge;

interface
uses
 windows, messages, sysutils, classes, controls, Graphics;

type

  TAnotherGauge = Class( TCustomControl )
  private
    FMin, FMax, FLow, FHigh, FValue, FBlockHeight :Integer;
    FForeColor, FLowColor, FHighColor :TColor;
    FOnCHange :TNotifyEvent;
    procedure SetValue( V :Integer );
    procedure SetLow( V :Integer );
    procedure SetHigh( V :Integer );
    procedure SetMin( V :Integer );
    procedure SetMax( V :Integer );
    procedure SetForeColor( C :TColor );
    procedure SetLowColor( C :TColor );
    procedure SetHighColor( C :TColor );
    procedure SetBlockHeight( V :Integer );
  protected
    procedure Change; virtual;
    procedure Paint; override;
  public
    constructor Create( anOwner :TComponent ); override;
  published
    property BlockHeight :Integer
      read FBlockHeight write SetBlockHeight;
    property Min :Integer
      read FMin write SetMin;
    property Max :integer
      read FMax write SetMax;
    property Value :Integer
      read FValue write SetValue;
    property Low :Integer
      read FLow write SetLow;
    property High :Integer
      read FHigh write SetHigh;
    property ForeColor :TColor
      read FForeColor write SetForeColor;
    property LowColor :TColor
      read FLowColor write SetLowColor;
    property HighColor :TColor
      read FHighCOlor write SetHighColor;
    property OnChange :TNotifyEvent
      read FOnCHange write FOnChange;
    property Color;
    property OnDblClick;
    property OnClick;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
  end;

procedure Register;

implementation

{$INCLUDE SYSUTILS.INC}

const

  DefGaugeWIdth = 50;
  DefGaugeHeight = 150;
  DefBlockHeight = 10;

  GridWidth = 10;

procedure Register;
begin
 RegisterComponents( 'more...', [ TAnotherGauge ] );
end;

procedure RangeError;
begin
 raise ERangeError.CreateRes( SRangeError );
end;


constructor TAnotherGauge.Create( anOwner :TComponent );
begin
  inherited Create( anOwner );
  Width := DefGaugeWidth;
  Height := DefGaugeHeight;
  FMin := 0;
  FMax := 100;
  FLow := 10;
  FHigh := 80;
  FValue := 50;
  FForeColor := clLime;
  FHighColor := clRed;
  FLowColor := clGreen;
  FBlockHeight := DefBlockHeight;
  Color := clBlack;
end;

procedure TAnotherGauge.SetValue( V :Integer );
Var R :TRect; C :Integer;
begin
  if ( csLoading in ComponentState ) then FValue := V
   else begin
     if ( V > FMax ) or ( V < FMin ) then RangeError;
     C := round( ( height * ( value - min )/( max - min ) ) / BlockHeight );
     R := Rect( 0, 0, ClientWidth - GridWidth, Height - C * BlockHeight );
     if C <> round( ( height * ( v - min )/( max - min ) ) / BlockHeight )
       then invalidateRect( Handle, @R, false );
     FValue := V;
     Change;
   end;
end;

procedure TAnotherGauge.SetLow( V :Integer );
begin
  if ( csLoading in ComponentState ) then FLow := V
   else begin
     if ( V > FHigh ) or ( V < FMin ) then RangeError;
     FLow := V;
     Invalidate;
   end;
end;

procedure TAnotherGauge.SetHigh( V :Integer );
begin
  if ( csLoading in ComponentState ) then FHigh := V
   else begin
     if ( V < FLow ) or ( V > FMax ) then RangeError;
     FHigh := V;
     Invalidate;
   end;
end;

procedure TAnotherGauge.SetMin( V :Integer );
begin
  if ( csLoading in ComponentState ) then FMin := V
   else begin
     if ( V > FMax ) or ( FValue < V ) then RangeError;
     FMin := V;
     if ( FLow < FMin ) then FLow := FMin;
     Invalidate;
   end;
end;

procedure TAnotherGauge.SetMax( V :Integer );
begin
  if ( csLoading in ComponentState ) then FMax := V
   else begin
     if ( V < FMin ) or ( FValue > V ) then RangeError;
     FMax := V;
     if ( FHigh > FMax ) then FHigh := FMax;
     Invalidate;
   end;
end;

procedure TAnotherGauge.SetForeColor( C :TColor );
begin
 FForeColor := C;
 Invalidate;
end;

procedure TAnotherGauge.SetLowColor( C :TColor );
begin
 FLowColor := C;
 Invalidate;
end;

procedure TAnotherGauge.SetHighColor( C :TColor );
begin
 FHighColor := C;
 Invalidate;
end;

procedure TAnotherGauge.SetBlockHeight( V :Integer );
begin
  if ( csLoading in ComponentState ) then FBlockHeight := V
   else begin
     if ( V > (( FMax - FMin ) div 4 )) or ( V <= 0 ) then RangeError;
     FBlockHeight := V;
     Invalidate;
   end;
end;


procedure TAnotherGauge.Change;
begin
 if assigned( FOnChange ) then FOnChange( Self );
end;

procedure TAnotherGauge.Paint;
var
 h, w, c, b, t, g :integer;
 pp :array[ 0..3 ] of TPoint;
begin
with Canvas do begin

 Brush.Color := Color;
 FillRect( CLientRect );

 h := ClientHeight;
 w := ClientWidth;
 g := w - GridWidth;

 c := h - round( ( h * ( value - min )/( max - min ) ) / BlockHeight ) * BlockHeight;

 if ( value > high ) then
   begin
    t := h - round( ( h * ( high - min )/( max- min ) )/ BlockHeight ) * BlockHeight;
    brush.color := FHighColor;
    fillrect( rect( 0, c, g, t ) );
   end
  else t := c;

 if ( value  > low ) then
   begin
    b := h - round( ( h * ( low - min ) / ( max - min ) ) / BlockHeight ) * BlockHeight;
    brush.color := LowColor;
    fillrect( rect( 0, b, g, h ) );
    brush.color := ForeColor;
    fillrect( rect( 0, t, g, b ) );
   end
  else
   begin
    b := h;
    brush.color := LowColor;
    fillrect( rect( 0, t, g, b ) );
   end;

 Pen.Color := Color;
 b := h;
 for t := 1 to ( ( h - c ) div BlockHeight ) do
  begin
   dec( b, BlockHeight );
   MoveTo( 0, b );
   LineTo( g, b );
  end;

 Pen.Color := ForeColor;
 pp[ 0 ] := point( w, 0 );
 pp[ 1 ] := point( g + 3, 0 );
 pp[ 2 ] := point( g + 3, h - 1 );
 pp[ 3 ] := point( w, h - 1 );
 PolyLine( pp );
 for t := 1 to ( h div BlockHeight ) do
   begin
     b := h - t * BlockHeight;
     MoveTo( g + 3, b );
     LineTo( w - 3, b );
   end;

 end;
end;



end.


